From 9de80c0f4b37272ca255fef91b968d63c523ddfd Mon Sep 17 00:00:00 2001
From: Matej Horvat <matej.horvat@guest.arnes.si>
Date: Mon, 24 Sep 2018 01:45:07 +0100
Subject: [PATCH] Load FFmpeg libraries in a saner and safer way

---
 .../platforms/ffmpeg/FFmpegRuntimeLinker.cpp  | 115 ++++++++----------
 1 file changed, 52 insertions(+), 63 deletions(-)

diff --git a/dom/media/platforms/ffmpeg/FFmpegRuntimeLinker.cpp b/dom/media/platforms/ffmpeg/FFmpegRuntimeLinker.cpp
index c4842bb96..aab67ad3a 100644
--- a/dom/media/platforms/ffmpeg/FFmpegRuntimeLinker.cpp
+++ b/dom/media/platforms/ffmpeg/FFmpegRuntimeLinker.cpp
@@ -66,75 +66,64 @@ FFmpegRuntimeLinker::Link()
   MOZ_ASSERT(NS_IsMainThread());
 
 #if defined(XP_WIN)
-  HKEY aKey;
-  DWORD d;
-  char* dllpath = new char[1000];
-
-  if(::RegOpenKeyExA(HKEY_CLASSES_ROOT,
-    "CLSID\\{171252A0-8820-4AFE-9DF8-5C92B2D66B04}\\InprocServer32",
-     0,KEY_READ, &aKey)!=0) return false;
-
-  if(::RegQueryValueExA(aKey, NULL, NULL, NULL, (LPBYTE)dllpath, &d)!=0) return false;
-
-  ::RegCloseKey(aKey);
-
-  int i = (int)d;
-	do{
-		i--;
-	} while (dllpath[i]!='\\'&&i>0);
-  i++;
-  ::lstrcpyA(&dllpath[i], "avcodec-lav-57.dll");
-  avcd=::LoadLibraryExA(dllpath,0,LOAD_WITH_ALTERED_SEARCH_PATH);
-  ::lstrcpyA(&dllpath[i], "avutil-lav-55.dll");
-  avutl=::LoadLibraryExA(dllpath,0,LOAD_WITH_ALTERED_SEARCH_PATH);
-
-  if(avcd==NULL||avutl==NULL){
-     delete [] dllpath;
-     return false;
-     } 
+  avcd = ::LoadLibraryExA("avcodec-57.dll", 0, LOAD_WITH_ALTERED_SEARCH_PATH);
+  if(!avcd)
+    return false;
 
-  HMODULE hmod=avcd;
+  avutl = ::LoadLibraryExA("avutil-55.dll", 0, LOAD_WITH_ALTERED_SEARCH_PATH);
+  if(!avutl)
+  {
+    ::FreeLibrary(avcd);
+    return false;
+  }
 
-  const char* avc_symbs[]={
-  "avcodec_version",
-  "avcodec_alloc_context3",
-  "avcodec_close",
-  "avcodec_decode_audio4",
-  "avcodec_decode_video2",
-  "avcodec_find_decoder",
-  "avcodec_flush_buffers",
-  "avcodec_open2",
-  "avcodec_register_all",
-  "av_init_packet",
-  "av_parser_init",
-  "av_parser_close",
-  "av_parser_parse2",
-  /* libavutil */
-  "av_log_set_level",
-  "av_freep",
-  "av_frame_alloc",
-  "av_frame_free",
-  "av_frame_unref"
+  static const char* avc_symbs[] =
+  {
+    // libavcodec
+    "avcodec_version",
+    "avcodec_alloc_context3",
+    "avcodec_close",
+    "avcodec_decode_audio4",
+    "avcodec_decode_video2",
+    "avcodec_find_decoder",
+    "avcodec_flush_buffers",
+    "avcodec_open2",
+    "avcodec_register_all",
+    "av_init_packet",
+    "av_parser_init",
+    "av_parser_close",
+    "av_parser_parse2",
+    // libavutil
+    "av_log_set_level",
+    "av_freep",
+    "av_frame_alloc",
+    "av_frame_free",
+    "av_frame_unref"
   };
 
-  i=0;
+  HMODULE hmod = avcd;
+  for(int i = 0; i < avc_symbs_count; i++)
+  {
+    if(i == _log_set_level)
+      hmod = avutl;
+
+    avc_ptr[i] = GetProcAddress(hmod, avc_symbs[i]);
+    if(!avc_ptr[i])
+    {
+      FFMPEG_LOG(
+        "Missing function ",
+        avc_symbs[i],
+        " in ",
+        (i < _log_set_level) ? "libavcodec" : "libavutil"
+      );
+
+      ::FreeLibrary(avcd);
+      ::FreeLibrary(avutl);
 
-  do{
-  if (i==13) hmod=avutl;
-    avc_ptr[i] = GetProcAddress(hmod,avc_symbs[i]);
-    if (avc_ptr[i]==NULL){
-      ::GetModuleFileNameA(hmod,dllpath,1000);
-      d=::lstrlenA(dllpath);
-  	do{
-		d--;
-	} while (dllpath[i]!='\\'&&i>0);
-      FFMPEG_LOG("Couldn't load function ",avc_symbs[i]," from %s.", &dllpath[d]);
-      delete [] dllpath;
       return false;
-     }
-    i++;
-  }while(i<avc_symbs_count);
-  delete [] dllpath;
+    }
+  }
+
   avcodec_version = (decltype(avcodec_version))avc_ptr[0];
   sLinkStatus = LinkStatus_SUCCEEDED;
   return true;
-- 
2.19.0.windows.1

